import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.LinkedList;

import javax.imageio.ImageIO;

import ag.debug.Debugger;


public class ScaleImages {
	
	public static void main(String[] args){
		String inputfolder = "/Users/alex/Desktop/GenomeImages/Images/";
		String outputfolder = "/Users/alex/Desktop/GenomeImages/scaled_images_700_percent/";
		int percentage = 700;
		String[] files = getFilesInFolder(inputfolder, "jpg");
		for(int i = 0; i < files.length; i++){
			try{
				String filename = files[i];
				scale(inputfolder+filename,percentage,outputfolder+filename);
				if(i % 10 == 0){
					Debugger.Debug("Scaled "+(i+1)+" images with "+percentage+" %");
				}
			}catch(Exception e){
				e.printStackTrace();
			}
		
		}
		
		
	}
	private static String[] getFilesInFolder(String folder,String extension){
		File myDir = new File(folder);
		LinkedList<String> filenames = new LinkedList<String>();
		if( myDir.exists() && myDir.isDirectory()){
			File[] files  = myDir.listFiles();
			for(int i=0; i < files.length; i++){
				File tmp = files[i];
				String filename = tmp.getName();
				if(filename.toLowerCase().contains("."+extension.toLowerCase())){
					filenames.add(filename);
				}
			}
		}
		String[] retval = new String[filenames.size()];
		for(int i = 0; i < retval.length; i++){
			retval[i] = filenames.get(i);
		}
		return retval;
	}
	//Scales an image(src) and writes it to dest
	public static void scale(String src, int width, int height, String dest) throws IOException {
		BufferedImage imgBefore = ImageIO.read(new File(src));
		BufferedImage imgAfter = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
		Graphics2D g = imgAfter.createGraphics();
		AffineTransform at = AffineTransform.getScaleInstance((double)width/imgBefore.getWidth(),(double)height/imgBefore.getHeight());
		g.drawRenderedImage(imgBefore,at);
		ImageIO.write(imgAfter,"JPG",new File(dest));
	}
	//Scales an image(src) with percentage and writes it to dest
	public static void scale(String src, int percentage, String dest) throws IOException{
		BufferedImage imgBefore = ImageIO.read(new File(src));
		int oldHeight = imgBefore.getHeight();
		int oldWidth = imgBefore.getWidth();
		int newHeight = (int)Math.round(oldHeight*(percentage/100));
		int newWidth = (int)Math.round(oldWidth*(percentage/100));
		scale(src,newWidth,newHeight,dest);
	}
	public static void scale(String src, double percentage, String dest)throws IOException{
		scale(src,(int)(percentage*100),dest);
	}
}
