package ag.genomeencoder.analyzer;

import java.io.File;
import java.io.FileNotFoundException;
import java.math.BigInteger;
import java.util.LinkedList;
import java.util.Scanner;
import ag.debug.*;
import ag.genomeencoder.encoder.GenomeRecord;
import ag.genomeencoder.encoder.DatabankParser.ParseKeyword;

public class EMBLAnalyzer {

	 public enum ParseKeyword { 
	        Identification,
	        AccesionNumber,
	        ProjectIdentifier,
	        Date,
	        Description,
	        Keyword,
	        OrganismSpecies,
	        OrganismClassification,
	        Organelle,
	        ReferenceNumber,
	        ReferenceComment,
	        ReferencePosition,
	        ReferenceCrossReference,
	        ReferenceGroup,
	        ReferenceAuthors,
	        ReferenceTitle,
	        ReferenceLocation,
	        DatabaseCrossReference,
	        CommentsOrNotes,
	        AssemblyHeader,
	        AssemblyInformation,
	        FeatureTableHeader,
	        FeatureTableData,
	        SpacerLine,
	        SequenceHeader,
	        ConstructLine,
	        SequenceData,
	        BlankSequenceData,
	        TerminationLine,
	        Empty
	    }
		private LinkedList<int[]> items;
		private String filename;
		public EMBLAnalyzer(String filename){
		    this.filename = filename;
		}
		public void analyzeLength(int numberOfElements)throws FileNotFoundException{
		    File databank = new File(filename);
		    items = new LinkedList<int[]>();
		    Scanner scanner = new Scanner(databank);
		    
		    try {
		      
	             String state;
	             int counter = 0;
	             GenomeRecord current = new GenomeRecord("EMPTY");
	             while (scanner.hasNextLine()){
	                 String line = scanner.nextLine();
	               
	                 state = line.substring(0,2);
	                 switch(getState(state)){
	                     case Identification:
	                    	int[] tmp = new int[1];
	                    	current.sequenceData = current.sequenceData.replace(" ", "");
	                    	tmp[0] = current.sequenceData.length();
	                    	items.add(tmp);
	                    	current = null;
	                        current = new GenomeRecord(line.substring(5));
	                        counter ++;
	                        break;
	                	case SequenceData:
	                	    line = line.substring(5);
	                	    line = line.replaceAll("[0-9]","");
	                    	current.sequenceData += line;
	                	    break;
	                	case BlankSequenceData:
	                		current.sequenceData += "\n";
	                	    break; 
	                 }
	             }
	            }
	            catch(Exception e) {
	              //ensure the underlying stream is always closed
	              System.out.println("Error:"+e.toString());
	            }
	            finally{
	               scanner.close(); 
	            }
		}
		public void analyzeLengthOptimize(int numberOfElements)throws FileNotFoundException{
		    File databank = new File(filename);
		    items = new LinkedList<int[]>();
		    Scanner scanner = new Scanner(databank);
		    int counter = 0;
		    int prevcounter = 1;
		    boolean truncateLength = !(numberOfElements == -1);
		    Debugger.Debug("Analyzing "+numberOfElements+" elements");
		    Debugger.Debug("Truncate length is:"+truncateLength);
		    
		    try {
		      
	             String state;
	             
	             GenomeRecord current = new GenomeRecord("EMPTY");
	             while (scanner.hasNextLine() && (truncateLength && numberOfElements > counter)){
	                 String line = scanner.nextLine();
	                 state = line.substring(0,2);
	                 if(state.equals("//")){
	                     //seq.add(current);
	                 }else if(state.equals("ID")){
	                	 int[] tmp = new int[1];
	                	 current.sequenceData = current.sequenceData.replace(" ", "");
	                	 tmp[0] = current.sequenceData.length();
	                	 items.add(tmp);
	                	 current = new GenomeRecord(line.substring(5));
	                	 counter ++;
	                 }
	                 else if(state.equals("  ")){
	                    line = line.substring(5);
	             	    line = line.replaceAll("[0-9]","");
	                 	current.sequenceData += line;
	                 }
	                 else if(state.equals("bb")){
	                     current.sequenceData += "\n";
	                  }
	               
	               if(counter % 100 == 0){
	                    if(prevcounter != counter){
	                        System.out.println("Number of records currently processed:"+counter);
	                        prevcounter = counter;
	                    }
	               }
	            }
	             
	            }
	            
	            catch(Exception e) {
	              //ensure the underlying stream is always closed
	              System.out.println("Error:"+e.toString());
	            }
	            finally{
	               scanner.close(); 
	            }
	            
		}
		public double getAverageLength(){
			String kameleon = "test";
			System.out.println(items.size());
			int length = (int)Math.ceil(((((items.size()/2)/2)/2)/2));
			length = 2;
			double[] averages = new double[(int)Math.ceil(items.size()/length)];
			int arraycounter = 0;
			int sum = 0;
			double retval = 0;
			for(int i = 0; i < items.size();i++){
				sum += items.get(i)[0];
				if(i%length ==  0 && i != 0){
					averages[arraycounter] = sum/length;;
					sum = 0;
					arraycounter ++;
				}
			}
			
			for(int i = 0; i < averages.length;i++){
				retval +=  averages[i];
			}
			retval = (retval/averages.length);
			return retval;
		}
		public double doubleGetAverageLengthWithException(){
			double sum = 0;
			LinkedList<double[]> sums = new LinkedList<double[]>();
			int tmpsum = 0;
			double itemcounter = 0;
			for(int i = 0; i < items.size();i++){
				tmpsum += items.get(i)[0];
				itemcounter ++;
				if(checkForOverflow(tmpsum)){
					tmpsum -= items.get(i)[0];
					double[] tmp = new double[2];
					tmp[0] = (double)tmpsum;
					tmp[1] = itemcounter;
					itemcounter = 0;
					sums.add(tmp);
					tmpsum = items.get(i)[0];
					Debugger.Debug("Entered overflow detection");
				}
			}
			Debugger.Debug("sum:"+tmpsum);
			double tmps = 0;
			int counter = 0;
			if(sums.size() > 0){
				for(int i = 0; i < sums.size();i++){
					tmps += sums.get(i)[0]/sums.get(i)[1];
					counter ++;
				}
				sum = tmps/counter;
			}
			else{
				sum = tmpsum/itemcounter;
			}
			return sum;
		}
		private boolean checkForOverflow(int tmpsum) {
			
			return tmpsum < 0;
		}
		public double getAverageLength2(){
			//TODO: verifiser denne
			double retval = 0;
			int length = (int)Math.ceil((double)items.size()/10);
			double[] averages = new double[length];
			double sum = 0;
			int tmpi = 0;
			int lasti = 0;
			int avgCounter = 0;
			Debugger.Debug("Size of list:"+items.size());
			for(int i = 0; i < items.size();i++){
				sum += items.get(i)[0];
				if(i % length == 0 && i != 0){
					averages[avgCounter] = sum/length;
					sum = 0;
					avgCounter ++;
					lasti = i;
				}
				tmpi = i;
			}
			
			Debugger.Debug(averages);
			if(sum != 0){
				double tmpsum = 0;
				for(int i = lasti; i < items.size();i++){
					tmpsum += items.get(i)[0];
				}
				averages[length-1] = tmpsum/(items.size()-lasti);
			}
			Debugger.Debug(averages);
			sum = 0;
			for(int i = 0; i < averages.length; i++){
				sum += averages[i];
			}
			sum = sum / averages.length;
			return sum;
		}
		public double testGetAverageLength(){
			double retval = 0;
			for(int i = 0; i < items.size();i++){
				retval += items.get(i)[0];
				
			}
			retval = retval/items.size();
			return retval;
		}
		public ParseKeyword getState(String state){
		    if(state.equals("//")){
		        //RESET STATE
		        return ParseKeyword.TerminationLine;
		    }
		    else if(state.equals("ID")){
		        //ID
		        return ParseKeyword.Identification;
		    }
		    else if(state.equals("XX")){
		        //newline
		        return ParseKeyword.SpacerLine;
		    }
		    else if(state.equals("AC")){
		        //accession number
		        return ParseKeyword.AccesionNumber;
		    }
		    else if(state.equals("PR")){
		        //Project identifier
		        return ParseKeyword.ProjectIdentifier;
		    }
		    else if(state.equals("DT")){
		        //Date
		        return ParseKeyword.Date;
		    }
		    else if(state.equals("DE")){
		        //Description
		        return ParseKeyword.Description;
		    }
		    else if(state.equals("KW")){
		        //keyword
		        return ParseKeyword.Keyword;
		    }
		    else if(state.equals("OS")){
		        //Organism Species
		        return ParseKeyword.OrganismSpecies;
		    }
	        else if(state.equals("OC")){
		        //Organism Classification
		        return ParseKeyword.OrganismClassification;
		    }
		    else if(state.equals("OG")){
		        //Organelle
		        return ParseKeyword.Organelle;
		    }
	        else if(state.equals("RN")){
		        //reference number
		        return ParseKeyword.ReferenceNumber;
		    }
		    else if(state.equals("RC")){
		        return ParseKeyword.ReferenceComment;
		    }
	        else if(state.equals("RP")){
		        //Reference page ?
		        return ParseKeyword.ReferencePosition;
		    }
	        else if(state.equals("RX")){
		        //Reference Identifcator
		        return ParseKeyword.ReferenceCrossReference;
		    }
		    else if(state.equals("RG")){
		        return ParseKeyword.ReferenceGroup;
		    }
	        else if(state.equals("RA")){
		        //Reference People
		        return ParseKeyword.ReferenceAuthors;
		    }
	        else if(state.equals("RT")){
		        //reference lol
		        return ParseKeyword.ReferenceTitle;
		    }
		    else if(state.equals("RL")){
		        //Reference Location
		        return ParseKeyword.ReferenceLocation;
		    }
		    else if(state.equals("DR")){
		        //Document reference
		        return ParseKeyword.DatabaseCrossReference;
		    }
		    else if(state.equals("CC")){
		        //Comment
		        return ParseKeyword.CommentsOrNotes;
		    }
		    else if(state.equals("AH")){
		        //assembly header
		        return ParseKeyword.AssemblyHeader;
		    }
		    else if(state.equals("AS")){
		        return ParseKeyword.AssemblyInformation;
		    }
		    else if(state.equals("FH")){
		        //FUNCTION HEADER
		        return ParseKeyword.FeatureTableHeader;
		    }
		    else if(state.equals("FT")){
		        //FUNCTION TAXONOMY
		        return ParseKeyword.FeatureTableData;
		    }
		    else if(state.equals("SQ")){
		        return ParseKeyword.SequenceHeader;
		    }
		    else if(state.equals("  ")){
		        return ParseKeyword.SequenceData;
		    }
		    else if(state.equals("CO")){
		        return ParseKeyword.ConstructLine;
		    }
		    else if(state.equals("bb")){
		        return ParseKeyword.BlankSequenceData;
		    }
		    else {
		        //return empty state
		        return ParseKeyword.Empty;
		    }   
	        
		}
	
}
