package ag.genomeencoder.encoder;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.*;
import java.io.*;

import ag.debug.Debugger;
public class DatabankParser{
    public enum ParseKeyword { 
        Identification,
        AccesionNumber,
        ProjectIdentifier,
        Date,
        Description,
        Keyword,
        OrganismSpecies,
        OrganismClassification,
        Organelle,
        ReferenceNumber,
        ReferenceComment,
        ReferencePosition,
        ReferenceCrossReference,
        ReferenceGroup,
        ReferenceAuthors,
        ReferenceTitle,
        ReferenceLocation,
        DatabaseCrossReference,
        CommentsOrNotes,
        AssemblyHeader,
        AssemblyInformation,
        FeatureTableHeader,
        FeatureTableData,
        SpacerLine,
        SequenceHeader,
        ConstructLine,
        SequenceData,
        BlankSequenceData,
        TerminationLine,
        Empty
    }
	private LinkedList<GenomeRecord> seq;
	private String filename;
	public DatabankParser(String filename){
	    this.filename = filename;
	    seq = new LinkedList<GenomeRecord>();
         
	}
	public void parseFileToDatabase(int numberOfElements)throws FileNotFoundException, InstantiationException, IllegalAccessException, ClassNotFoundException, SQLException{
	    File databank = new File(filename);
	    Class.forName ("com.mysql.jdbc.Driver").newInstance();
	    Connection conn = DriverManager.getConnection ("jdbc:mysql://localhost:3306/genomedb","genomedb", "genomedb");
	    
	    Scanner scanner = new Scanner(databank);
	  
	    try {
	      
             String state;
             int counter = 0;
             GenomeRecord current = new GenomeRecord("EMPTY");
             while (scanner.hasNextLine()){
                 String line = scanner.nextLine();
                 if(counter % 100 == 0){
                	 Debugger.Debug("Parsed "+counter+" elements");
                 }
                 state = line.substring(0,2);
                 switch(getState(state)){
                     case TerminationLine:
                        //seq.add(current);
                    	 current.sequenceData = current.sequenceData.replace("   ", " ");
                    	 current.sequenceData = current.sequenceData.replace("  ", " ");
                    	 current.sequenceData = current.sequenceData.replace("  ", " ");
                    	 current.sequenceData = current.sequenceData.replace("\n", " ");
                        String query = "INSERT INTO genomeEntity VALUES(NULL,'"+current.identification+"','"+current.accesionNumber+"','"+current.projectIdentifier+"','"+current.date+"','"+current.description+"','"+current.keyword+"','"+current.organismSpecies+"','"+current.organismClassification+"','"+current.organelle+"','"+current.referenceNumber+"','"+current.referenceComment+"','"+current.referencePosition+"','"+current.referenceCrossReference+"','"+current.referenceGroup+"','"+current.referenceAuthors+"','"+current.referenceTitle+"','"+current.referenceTitle+"','"+current.databaseCrossReference+"','"+current.commentsOrNotes+"','"+current.assemblyHeader+"','"+current.assemblyInformation+"','"+current.featureTableHeader+"','"+current.featureTableData+"', '"+current.sequenceHeader+"','"+current.sequenceData+"');";
                        Statement stat = conn.createStatement();
                        stat.execute(query);
                        break;
                     case Identification:
                        current = new GenomeRecord(line.substring(5));
                        counter ++;
                        break;
                	case AccesionNumber:
                    	current.accesionNumber += line.substring(5);
                	    break;
                	case ProjectIdentifier:
                    	current.projectIdentifier+= line.substring(5);
                	    break;
                	case Date:
                    	current.date += line.substring(5);
                	    break;
                	case Description:
                    	current.description += line.substring(5);
                	    break;
                	case Keyword:
                    	current.keyword += line.substring(5);
                	    break;
                	case OrganismSpecies:
                    	current.organismSpecies += line.substring(5);
                	    break;
                	case OrganismClassification:
                    	current.organismClassification += line.substring(5);
                	    break;
                	case Organelle:
                    	current.organelle += line.substring(5);
                	    break;
                	case ReferenceNumber:
                    	current.referenceNumber += line.substring(5);
                	    break;
                	case ReferenceComment:
                    	current.referenceComment += line.substring(5);
                	    break;
                	case ReferencePosition:
                    	current.referencePosition += line.substring(5);
                	    break;
                	case ReferenceCrossReference:
                    	current.referenceCrossReference += line.substring(5);
                	    break;
                	case ReferenceGroup:
                    	current.referenceGroup += line.substring(5);
                	    break;
                	case ReferenceAuthors:
                    	current.referenceAuthors += line.substring(5);
                	    break;
                	case ReferenceTitle:
                    	current.referenceTitle += line.substring(5);
                	    break;
                	case ReferenceLocation:
                    	current.referenceLocation += line.substring(5);
                	    break;
                	case DatabaseCrossReference:
                    	current.databaseCrossReference += line.substring(5);
                	    break;
                	case CommentsOrNotes:
                    	current.commentsOrNotes += line.substring(5);
                	    break;
                	case AssemblyHeader:
                    	current.assemblyHeader += line.substring(5);
                	    break;
                	case AssemblyInformation:
                    	current.assemblyInformation += line.substring(5);
                	    break;
                	case FeatureTableHeader:
                	    if(line.length() > 5){
                    	    current.featureTableHeader += line.substring(5);
                    	}
                	    break;
                	case FeatureTableData:
                    	current.featureTableData += line.substring(5);
                	    break;
                	case SpacerLine:
                		current.spacerLine += line;
                	break;
                	case SequenceHeader:
                    	current.sequenceHeader += line.substring(5);
                	    break;
                	case ConstructLine:
                    	current.constructLine += line.substring(5);
                	    break;
                	case SequenceData:
                	    line = line.substring(5);
                	    line = line.replaceAll("[0-9]","");
                	   
                    	current.sequenceData += line;
                	    break;
                	case BlankSequenceData:
                		current.sequenceData += "\n";
                	    break; 
                 }
             }
            }
            catch(Exception e) {
              //ensure the underlying stream is always closed
              System.out.println("Error:"+e.toString());
            }
            finally{
               scanner.close();
               conn.close();
            }
	}
	public void parseFileMemoryEfficient(int numberOfElements)throws FileNotFoundException{
	    File databank = new File(filename);
	    
	    Scanner scanner = new Scanner(databank);
	    int counter = 0;
	    int prevcounter = 1;
	    try {
	      
             String state;
             
             GenomeRecord current = new GenomeRecord("EMPTY");
             while (scanner.hasNextLine() && counter <= numberOfElements){
                 String line = scanner.nextLine();
               
                 state = line.substring(0,2);
                 if(state.equals("//")){
                     seq.add(current);
                 }else if(state.equals("ID")){
                    current = new GenomeRecord(line.substring(5));
                    counter ++;
                 }
                 else if(state.equals("SQ")){
                    current.sequenceHeader += line.substring(5);
                 }
                 else if(state.equals("  ")){
                    line = line.substring(5);
             	    line = line.replaceAll("[0-9]","");
             	   
                 	current.sequenceData += line;
                 }
                 else if(state.equals("bb")){
                     current.sequenceData += "\n";
                  }
               
               if(counter % 100 == 0){
                    if(prevcounter != counter){
                        System.out.println("Number of records processed:"+counter);
                        prevcounter = counter;
                    }
               }
            }
            
            }
            
            catch(Exception e) {
              //ensure the underlying stream is always closed
              System.out.println("Error:"+e.toString());
            }
            finally{
               scanner.close(); 
            }
	}
	public ParseKeyword getState(String state){
	    if(state.equals("//")){
	        //RESET STATE
	        return ParseKeyword.TerminationLine;
	    }
	    else if(state.equals("ID")){
	        //ID
	        return ParseKeyword.Identification;
	    }
	    else if(state.equals("XX")){
	        //newline
	        return ParseKeyword.SpacerLine;
	    }
	    else if(state.equals("AC")){
	        //accession number
	        return ParseKeyword.AccesionNumber;
	    }
	    else if(state.equals("PR")){
	        //Project identifier
	        return ParseKeyword.ProjectIdentifier;
	    }
	    else if(state.equals("DT")){
	        //Date
	        return ParseKeyword.Date;
	    }
	    else if(state.equals("DE")){
	        //Description
	        return ParseKeyword.Description;
	    }
	    else if(state.equals("KW")){
	        //keyword
	        return ParseKeyword.Keyword;
	    }
	    else if(state.equals("OS")){
	        //Organism Species
	        return ParseKeyword.OrganismSpecies;
	    }
        else if(state.equals("OC")){
	        //Organism Classification
	        return ParseKeyword.OrganismClassification;
	    }
	    else if(state.equals("OG")){
	        //Organelle
	        return ParseKeyword.Organelle;
	    }
        else if(state.equals("RN")){
	        //reference number
	        return ParseKeyword.ReferenceNumber;
	    }
	    else if(state.equals("RC")){
	        return ParseKeyword.ReferenceComment;
	    }
        else if(state.equals("RP")){
	        //Reference page ?
	        return ParseKeyword.ReferencePosition;
	    }
        else if(state.equals("RX")){
	        //Reference Identifcator
	        return ParseKeyword.ReferenceCrossReference;
	    }
	    else if(state.equals("RG")){
	        return ParseKeyword.ReferenceGroup;
	    }
        else if(state.equals("RA")){
	        //Reference People
	        return ParseKeyword.ReferenceAuthors;
	    }
        else if(state.equals("RT")){
	        //reference lol
	        return ParseKeyword.ReferenceTitle;
	    }
	    else if(state.equals("RL")){
	        //Reference Location
	        return ParseKeyword.ReferenceLocation;
	    }
	    else if(state.equals("DR")){
	        //Document reference
	        return ParseKeyword.DatabaseCrossReference;
	    }
	    else if(state.equals("CC")){
	        //Comment
	        return ParseKeyword.CommentsOrNotes;
	    }
	    else if(state.equals("AH")){
	        //assembly header
	        return ParseKeyword.AssemblyHeader;
	    }
	    else if(state.equals("AS")){
	        return ParseKeyword.AssemblyInformation;
	    }
	    else if(state.equals("FH")){
	        //FUNCTION HEADER
	        return ParseKeyword.FeatureTableHeader;
	    }
	    else if(state.equals("FT")){
	        //FUNCTION TAXONOMY
	        return ParseKeyword.FeatureTableData;
	    }
	    else if(state.equals("SQ")){
	        return ParseKeyword.SequenceHeader;
	    }
	    else if(state.equals("  ")){
	        return ParseKeyword.SequenceData;
	    }
	    else if(state.equals("CO")){
	        return ParseKeyword.ConstructLine;
	    }
	    else if(state.equals("bb")){
	        return ParseKeyword.BlankSequenceData;
	    }
	    else {
	        //return empty state
	        return ParseKeyword.Empty;
	    }   
        
	}
	public LinkedList<GenomeRecord> getSequenceNames(){
		return seq;
	}
	public LinkedList<GenomeRecord> getSequences(){
		return seq;
	}
}
