package ag.genomeencoder.encoder;

/*
A	0000 0001
C   0000 0010
T   0000 0011
G	0000 0100

AA 	0001 0001 17
AC	0001 0010 18
AT	0001 0011 19
AG	0001 0100 20

CC	0010 0010 34
CA	0010 0001 33
CT	0010 0011 35
CG	0010 0100 36

TT	0011 0011 51
TA	0011 0001 49
TC	0011 0010 50
TG	0011 0100 52

GG	0100 0100 68
GA	0100 0001 65
GC	0100 0010 66
GT	0100 0011 67

 */
/*
 * Encodes 
 */
public class GenomeEntity {
	byte[] data;
	public static final int NUC_A = 0;
	public static final int NUC_C = 1;
	public static final int NUC_T = 2;
	public static final int NUC_G   = 3;
	public GenomeEntity(String sequence){
		int pairLength = (int) Math.ceil(sequence.length()/2);
		data = new byte[pairLength];
		int j = 0;
		for(int i = 0; i < pairLength; i++){
			j = i*2;
			if(sequence.length() >= j+1){
				data[i] = getBitmask(sequence.substring(j, j+1));
				data[i] = (byte) (data[i] << 4);
				if(sequence.length() >= j+2){
					data[i] = (byte) (data[i] | getBitmask(sequence.substring(j+1, j+2)));
				}
			}
		}
	}
	private byte getBitmask(String nuc){
		if(nuc.toLowerCase().equals("a")){
			return 1;
		}else if(nuc.toLowerCase().equals("c")){
			return 2;
		}else if(nuc.toLowerCase().equals("t")){
			return 3;
		}else if(nuc.toLowerCase().equals("g")){
			return 4;
		}
		return 0;
	}
	private String getNucleotide(byte data){

		if(data == 1){
			return "a";
		}else if(data == 2){
			return "c";
		}else if(data == 3){
			return "t";
		}else if(data == 4){
			return "g";
		}else{
			return "";
		}
		
	}
	private int getNucleotide1(byte data){
		if(data == 1){
			return NUC_A;
		}else if(data == 2){
			return NUC_C;
		}else if(data == 3){
			return NUC_T;
		}else if(data == 4){
			return NUC_G;
		}
		return -1;
		
	}
	public boolean containsNucleotide(int nucleotide){
		boolean retval = false;
		for(int i = 0; i < data.length;i++){
			byte one = (byte) ((byte)data[i] & 240);
			one = (byte) ((byte)one >> 4);
			byte two = (byte) ((byte)data[i] & 15);
			if(getNucleotide1(one) == nucleotide || getNucleotide1(two) == nucleotide){
				return true;
			}
		}
		return false;
	}
	public String decode(){
		String retval = "";
		for(int i = 0; i < data.length;i++){
			byte one = (byte) ((byte)data[i] & 240);
			one = (byte) ((byte)one >> 4);
			byte two = (byte) ((byte)data[i] & 15);
			retval += getNucleotide(one)+getNucleotide(two);
		}
		return retval;
	}
	public void printBinary(){
		for(int i = 0; i < data.length; i++){
			System.out.println(pad(Integer.toString(data[i], 2)));
		}
	}
	public void print(){
		for(int i = 0; i < data.length; i++){
			System.out.println(data[i]);
		}
	}
	public void printString(){
		System.out.println(this.decode());
	}
	public String toString(){
		return this.decode();
	}
	public String pad(String input){
		String output = input;
		for(int i = 0; i < 8-input.length();i++){
			output = "0"+output;
		}
		return output;
	}
	public byte[] getData(){
		return data;
	}
}
