package ag.genomeencoder.encoder;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Vector;

import ag.debug.Debugger;
import ag.genomeencoder.image.FFTAlgorithm;
import ag.genomeencoder.image.GenomeImage;
import ag.genomeencoder.image.ImageProcessingAlgorithm;
import ag.genomeencoder.image.SimpleAlgorithm;
public class SequenceUtilities {
	public LinkedList<GenomeSequence> createSequenceList(LinkedList<GenomeRecord> seq){
		LinkedList<GenomeSequence> list = new LinkedList<GenomeSequence>();
		for(int i = 0; i < seq.size();i++){
			list.add(createSequenceFromRecord(seq.get(i)));
		}
		return list;
		
	}
	public GenomeSequence createSequenceFromRecord(GenomeRecord rec){
		GenomeSequence seq = new GenomeSequence(rec.identification);
		seq.parseFromString(rec.sequenceData);
		return seq;
	}

	public int counteNumberOfUniqueStates(GenomeSequence seq){
		
		HashMap<String, Integer> map = new HashMap<String, Integer>();
		LinkedList<GenomeEntity> list = seq.list;
		for(int i = 0; i < list.size();i++){
			for(int j = 0; j < list.get(i).data.length;j++){
				map.put(list.get(i).data[j]+"", 1);
			}
		}
		return map.values().size();
	}
	public void dropSqlToImagesWithSimpleAlgorithm(String outputfolder, String limit){
		Connection conn = null;
		try {
			
			Class.forName ("com.mysql.jdbc.Driver").newInstance();
			String query = "select identification,sequence_data from genomeEntity "+limit+";";
			conn = DriverManager.getConnection ("jdbc:mysql://localhost:3306/genomedb","genomedb", "genomedb");
			PreparedStatement stat = conn.prepareStatement(
			        query,
			        ResultSet.TYPE_FORWARD_ONLY,
			        ResultSet.CONCUR_READ_ONLY);
			stat.setFetchSize(Integer.MIN_VALUE);
			ResultSet res = stat.executeQuery();
            int counter = 0;
			
			while(res.next()){
				String sequenceString = res.getString("sequence_data");
				String id = res.getString("identification");
				GenomeSequence seq = new GenomeSequence(id);
				seq.parseFromString(sequenceString);
				int widthandhight = (int) Math.ceil(Math.sqrt(seq.getList().size()));
	    		GenomeImage img = new GenomeImage(id,widthandhight,widthandhight);
	    		SimpleAlgorithm testalg = new SimpleAlgorithm(seq);
	    		img.Process(testalg);
				img.saveImage(outputfolder+id+"jpg");
				counter++;
				Debugger.Debug("Processed "+counter+" items");
			}
			res.close();


		} catch (Exception e) {
			e.printStackTrace();
		}finally{
			try {
				conn.close();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}
	public void dropRleSqlToImagesWithSimpleAlgorithm(String outputfolder, String limit) {
		Connection conn = null;
		try {
			
			Class.forName ("com.mysql.jdbc.Driver").newInstance();
			String query = "select identification,sequence_data from rle_genomeEntity "+limit+";";
			conn = DriverManager.getConnection ("jdbc:mysql://localhost:3306/genomedb","genomedb", "genomedb");
			PreparedStatement stat = conn.prepareStatement(
			        query,
			        ResultSet.TYPE_FORWARD_ONLY,
			        ResultSet.CONCUR_READ_ONLY);
			stat.setFetchSize(Integer.MIN_VALUE);
			ResultSet res = stat.executeQuery();
            int counter = 0;
			
			while(res.next()){
				String sequenceString = res.getString("sequence_data");
				String id = res.getString("identification");
				GenomeSequence seq = new GenomeSequence(id);
				seq.parseFromString(sequenceString);
				int widthandhight = (int) Math.ceil(Math.sqrt(seq.getList().size()));
	    		GenomeImage img = new GenomeImage(id,widthandhight,widthandhight);
	    		SimpleAlgorithm testalg = new SimpleAlgorithm(seq);
	    		img.Process(testalg);
				img.saveImage(outputfolder+id+"jpg");
				counter++;
				Debugger.Debug("Processed "+counter+" items");
			}
			res.close();


		} catch (Exception e) {
			e.printStackTrace();
		}finally{
			try {
				conn.close();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		
	}
	public void dumpAlignedImages(String outputfolder, String limit){
		Connection conn = null;
		try {
			
			Class.forName ("com.mysql.jdbc.Driver").newInstance();
			String query = "select identification,sequence_data from rle_genomeEntity "+limit+";";
			conn = DriverManager.getConnection ("jdbc:mysql://localhost:3306/genomedb","genomedb", "genomedb");
			PreparedStatement stat = conn.prepareStatement(
			        query,
			        ResultSet.TYPE_FORWARD_ONLY,
			        ResultSet.CONCUR_READ_ONLY);
			stat.setFetchSize(Integer.MIN_VALUE);
			ResultSet res = stat.executeQuery();
            int counter = 0;
			
			while(res.next()){
				String sequenceString = res.getString("sequence_data");
				String id = res.getString("identification");
				String[] alignments = createStringAlignments(sequenceString,900);
				Debugger.Debug(alignments.length+"");
				for(int i = 0; i < alignments.length; i++){
					if(alignments[i].length() > 0){
						GenomeSequence seq = new GenomeSequence(id+"no"+i);
						seq.parseFromString(alignments[i]);
						int widthandheight = (int) Math.ceil(Math.sqrt(seq.getList().size()));
						
			    		GenomeImage img = new GenomeImage(id,widthandheight,widthandheight);
			    		SimpleAlgorithm testalg = new SimpleAlgorithm(seq);
			    		img.Process(testalg);
						img.saveImage(outputfolder+id+"no"+i+"jpg");
					}
				}
				
				counter++;
				Debugger.Debug("Processed "+counter+" items");
			}
			res.close();


		} catch (Exception e) {
			e.printStackTrace();
		}finally{
			try {
				conn.close();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		
	}
	public static String[] createStringAlignments(String input, int length){
		if(length > input.length()){
				String[] tmp = new String[1];
				tmp[0] = input;
				return tmp;
		}
		int numberOfElements = input.length();
		String[] alignments = new String[numberOfElements];
		int storei = 0;
		for(int i = 0; i < numberOfElements; i++){
			try{
				alignments[i] = input.substring(i,length+i);
			}catch(Exception e){
				alignments[i] = input.substring(i,numberOfElements-1);
				int remainder = length - alignments[i].length();
				alignments[i] += input.substring(0,remainder);
			}
			storei = i;
		}
		alignments[alignments.length -1] = input.substring(storei, input.length()-1);
		return alignments;
	}
	public void dropGenomeEntityToImageWithFFT(String outputfolder, String limit) {
		Connection conn = null;
		try {
			
			Class.forName ("com.mysql.jdbc.Driver").newInstance();
			String query = "select identification,sequence_data from genomeEntity "+limit+";";
			conn = DriverManager.getConnection ("jdbc:mysql://localhost:3306/genomedb","genomedb", "genomedb");
			PreparedStatement stat = conn.prepareStatement(
			        query,
			        ResultSet.TYPE_FORWARD_ONLY,
			        ResultSet.CONCUR_READ_ONLY);
			stat.setFetchSize(Integer.MIN_VALUE);
			ResultSet res = stat.executeQuery();
            int counter = 0;
			
			while(res.next()){
				String sequenceString = res.getString("sequence_data");
				String id = res.getString("identification");
				GenomeSequence seq = new GenomeSequence(id);
				seq.parseFromString(sequenceString);
				int widthandhight = (int) Math.ceil(Math.sqrt(seq.getList().size()));
	    		GenomeImage img = new GenomeImage(id,widthandhight,widthandhight);
	    		FFTAlgorithm testalg = new FFTAlgorithm(seq);
	    		try{
	    			img.Process(testalg);
	    			img.saveImage(outputfolder+id+"jpg");
	    		}catch(Exception e){
	    			Debugger.Debug("ERROR PROCESSING ITEM NO:"+counter);
	    		}
	    		counter++;
				Debugger.Debug("Processed "+counter+" items");
				img = null;
				seq = null;
				System.gc();
			}
			res.close();


		} catch (Exception e) {
			e.printStackTrace();
		}finally{
			try {
				conn.close();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		
	}
}
