package ag.lirebenchmark;

import java.awt.image.BufferedImage;
import java.io.IOException;

import net.semanticmetadata.lire.DocumentBuilder;
import net.semanticmetadata.lire.ImageSearchHits;
import net.semanticmetadata.lire.ImageSearcher;
import net.semanticmetadata.lire.ImageSearcherFactory;

import org.apache.lucene.index.IndexReader;

public class LireGenomeIndex {
	private String indexpath;
	private PreProcessingAlgorithm alg;
	private IndexReader reader;
	private float colorHistogramWeight = 0.1f;
	private float colorDistributionWeight = 0.1f;
	private float textureWeight = 1.0f;
	private int numberOfHits = 100;
	
	public LireGenomeIndex(String path, PreProcessingAlgorithm algorithm){
		indexpath = path; 
		alg = algorithm;
	}
	public LireGenomeIndex(String path, PreProcessingAlgorithm algorithm, float histogram, float distribution, float texture){
		indexpath = path; 
		alg = algorithm;
		colorHistogramWeight = histogram;
		colorDistributionWeight = distribution;
		textureWeight = texture;
	}
	public void loadIndex()throws IOException{
		reader = IndexReader.open(indexpath);
	}
	public String[] search(String query){
		String[] results = null;
		BufferedImage queryImage;
		ImageSearcher searcher;
		ImageSearchHits hits;
		queryImage = alg.processInput(query);
		searcher = ImageSearcherFactory.createWeightedSearcher(numberOfHits, colorHistogramWeight, colorDistributionWeight, textureWeight);
		hits = null;
		try {
			hits = searcher.search(queryImage, reader);
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		results = new String[hits.length()];
		if(numberOfHits > hits.length()){
			numberOfHits = hits.length();
		}
		for (int i = 0; i < numberOfHits; i++) {
			 results[i] = hits.score(i) + " - "+ hits.doc(i).getField(DocumentBuilder.FIELD_NAME_IDENTIFIER).stringValue(); 
		}
		
		return results;
	}
}
