package ag.liregui;

import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;

import javax.imageio.ImageIO;
import javax.swing.DefaultComboBoxModel;
import javax.swing.ListModel;

import net.semanticmetadata.lire.DocumentBuilder;
import net.semanticmetadata.lire.ImageSearchHits;
import net.semanticmetadata.lire.ImageSearcher;
import net.semanticmetadata.lire.ImageSearcherFactory;

import org.apache.lucene.document.Document;
import org.apache.lucene.index.IndexReader;

import com.mysql.jdbc.Statement;

import ag.debug.Debugger;
import ag.genomeencoder.encoder.GenomeSequence;
import ag.genomeencoder.image.FFTAlgorithm;
import ag.genomeencoder.image.GenomeImage;
import ag.genomeencoder.image.SimpleAlgorithm;
import ag.genomeenvoder.compressor.RunLengthEncoding;

public class SearchController {
	private ListModel resultModel;
	private IndexReader reader;
	private float colorHistogramWeight = 0.0f;
	private float colorDistributionWeight = 0.0f;
	private float textureWeight = 1.0f;
	Mainwindow owner;
	public SearchController(Mainwindow owner){
		this.owner = owner;
	}
	public SearchController(){
		
	}
	public boolean loadIndex(String url){
		boolean retval = true;
		try{
			reader = IndexReader.open(url);
		}catch(Exception e){
			retval = false;
			e.printStackTrace();
		}
		
		return retval;
	}
	public ListModel search(String input){
		BufferedImage img = convertSequenceToImage(input);
		return searchCore(img);
		
	}
	private ListModel searchCore(BufferedImage img){
		int numberOfHits = 15;
		try {
			ImageSearcher searcher = ImageSearcherFactory.createWeightedSearcher(numberOfHits, colorHistogramWeight, colorDistributionWeight, textureWeight);
			ImageSearchHits hits = null;
			hits = searcher.search(img, reader);
			String[] results = new String[hits.length()];
			
			if(numberOfHits > hits.length()){
				numberOfHits = hits.length();
			}
			for (int i = 0; i < numberOfHits; i++) {
				 results[i] = "#"+i+"|"+hits.score(i) + "|"+ hits.doc(i).getField(DocumentBuilder.FIELD_NAME_IDENTIFIER).stringValue();
			      
			}
			resultModel = new DefaultComboBoxModel(results);
		} catch (IOException e) {
			owner.setStatus("Error processing query");
			resultModel = new DefaultComboBoxModel(new String[]{});
		}
		
		return resultModel;
	}
	public ListModel search2(String input){
		//TODO: finn ut ka som skjer her
		String searchItem = writeSearchInputToImageFile(input);
		if(searchItem.equals("")){
			owner.setStatus("Query error");
			return new DefaultComboBoxModel(new String[]{});
		}
		try{
			Debugger.Debug("Setting up search...");
			owner.setStatus("Searching");
			
	        ImageSearcher searcher = ImageSearcherFactory.createDefaultSearcher();
	        FileInputStream imageStream = new FileInputStream(searchItem);
	        BufferedImage bimg = ImageIO.read(imageStream);
	        ImageSearchHits hits = null;
			hits = searcher.search(bimg, reader);
	        for (int i = 0; i < hits.length(); i++) {
	            System.out.println(hits.score(i) + ": " + 
					hits.doc(i).getField(DocumentBuilder.FIELD_NAME_IDENTIFIER).stringValue());
	        }
			Debugger.Debug("Searching...");
	        Document document = hits.doc(0);
			hits = searcher.search(document, reader);
			
			String[] results = new String[hits.length()];
	        for (int i = 0; i < hits.length(); i++) {
	        	if(hits.score(i) > 0.5){
	            results[i] = "#"+i+" - "+hits.score(i) + ": " + 
					hits.doc(i).getField(DocumentBuilder.FIELD_NAME_IDENTIFIER).stringValue();
	        	}
	        }
	        resultModel = new DefaultComboBoxModel(results);
		}catch(Exception e){
			owner.setStatus("Error occured while searching");
		}
		return resultModel;
	}
	
	//takes the input from the searchbox and creates a query image, returns imagepath
	private String writeSearchInputToImageFile(String input){
		String retval = "";
		GenomeSequence searchableSequence = new GenomeSequence("SEARCH_ITEM");
		try{
			searchableSequence.parseFromString(input);
			int widthandheight = (int) Math.ceil(Math.sqrt(searchableSequence.getList().size()));
			GenomeImage img = new GenomeImage(searchableSequence.getId(),widthandheight,widthandheight);
			SimpleAlgorithm alg = new SimpleAlgorithm(searchableSequence);
    		img.Process(alg);
    		retval = "SEARCH_ITEM"+Math.round(Math.random()*100)+"-"+Math.round(Math.random()*100)+"-"+Math.round(Math.random()*100)+"-"+Math.round(Math.random()*100)+"-"+Math.round(Math.random()*100)+"-";
    		img.saveImage(retval);
		}
		catch(Exception e){
			return "";
		}
		return retval;
	}
	public static BufferedImage convertSequenceToImage(String input){
		GenomeSequence searchableSequence = new GenomeSequence("SEARCH_ITEM");
		searchableSequence.parseFromString(input);
		int widthandheight = (int) Math.ceil(Math.sqrt(searchableSequence.getList().size()));
		GenomeImage img = new GenomeImage(searchableSequence.getId(),widthandheight,widthandheight);
		SimpleAlgorithm alg = new SimpleAlgorithm(searchableSequence);
		img.Process(alg);
		return img.getImage();
	}
	public static BufferedImage convertSequenceToImageFourier(String input){
		GenomeSequence searchableSequence = new GenomeSequence("SEARCH_ITEM");
		searchableSequence.parseFromString(input);
		int widthandheight = (int) Math.ceil(Math.sqrt(searchableSequence.getList().size()));
		GenomeImage img = new GenomeImage(searchableSequence.getId(),widthandheight,widthandheight);
		FFTAlgorithm alg = new FFTAlgorithm(searchableSequence);
		img.Process(alg);
		return img.getImage();
	}
	public ListModel searchRle(String input) {	
		RunLengthEncoding RLE = new RunLengthEncoding();
		return this.search(RLE.encode(input));
	}
	public ListModel searchScale(String input, int scalepercentage){
		BufferedImage img = convertSequenceToImage(input);
		img = scaleImage(img, scalepercentage);
		return searchCore(img);
	}
	public ListModel searchFourier(String input){
		BufferedImage img = convertSequenceToImageFourier(input);
		return searchCore(img);
	}
	public String returnSequenceDataFromId(String id, String table){
		String retval = "";
		Connection conn = null;
		try {
			
			Class.forName ("com.mysql.jdbc.Driver").newInstance();
			conn = DriverManager.getConnection ("jdbc:mysql://localhost:3306/genomedb","genomedb", "genomedb");
			java.sql.Statement stat = conn.createStatement();
			
			ResultSet res = stat.executeQuery("select sequence_data from "+table+" where id=(select table_id from genome_lookup where identification='"+id+"' limit 1)");
			while(res.next()){
				retval = res.getString("sequence_data");
			}
		}
		catch(Exception e){
			e.printStackTrace();
		}
		return retval;
	}
	public static BufferedImage scaleImage(BufferedImage in, int percentage){
		int newWidth = (int)Math.round(in.getWidth()*(percentage/100));
		int newHeight = (int)Math.round(in.getHeight()*(percentage/100));
		
		BufferedImage imgAfter = new BufferedImage(newWidth, newHeight, BufferedImage.TYPE_INT_RGB);
		Graphics2D g = imgAfter.createGraphics();
		AffineTransform at = AffineTransform.getScaleInstance((double)newWidth/in.getWidth(),(double)newHeight/in.getHeight());
		g.drawRenderedImage(in,at);
		in = null;
		return imgAfter;
	}
	public void setSettings(float colorHistogram, float colorDistribution, float texture){
		colorHistogramWeight = colorHistogram;
		colorDistributionWeight = colorDistribution;
		textureWeight = texture;
	}
	public double getHistogramSetting(){
		return colorHistogramWeight;
	}
	public double getDistributionSetting(){
		return colorDistributionWeight;
	}
	public double getTextureSetting(){
		return textureWeight;
	}
}

