package ag.genomeencoder.image;

import java.awt.Color;
import java.awt.image.BufferedImage;
import java.util.LinkedList;

import edu.emory.mathcs.jtransforms.fft.*;



import ag.debug.Debugger;
import ag.genomeencoder.encoder.GenomeEntity;
import ag.genomeencoder.encoder.GenomeSequence;

public class FFTAlgorithm implements ImageProcessingAlgorithm {
	GenomeSequence seq;
	int window = 400;
	public FFTAlgorithm(GenomeSequence sequence){
		this.seq = sequence;
	}
	
	public BufferedImage Process(BufferedImage image){
		LinkedList<Color> pixels = new LinkedList<Color>();
		float[][] res = generateBISOfWholeSequence();
		float[] a,c,t,g;
		a=null;
		c=null;
		t=null;
		g=null;
		FloatFFT_1D fft = new FloatFFT_1D(window);
		double awr,awg,awb = 1;
		double cwr,cwg,cwb = 1;
		double twr,twg,twb = 1;
		double gwr,gwg,gwb = 1;
		int length = res[0].length;
		//red vectors
		awr = 1;cwr = 10;twr = 1;gwr = 10;
		//green vectors
		awg = 1;cwg = 10;twg = 1;gwg = 10;
		//blue vectors
		awb = 1;cwb = 10;twb = 1;gwb = 10;
		int jepp = res[0].length-window;//res[0].length-(res[0].length%window);
		int numberofpixels = (int) Math.floor(((jepp+1)*window)/2);
		int sizeofimage = (int) Math.floor(Math.sqrt(numberofpixels));
		image = new BufferedImage(sizeofimage,sizeofimage,BufferedImage.TYPE_INT_RGB);
		int y = 0;int x = 0;
		for(int i = 0; i < jepp+1;i++){
			a = getSubBIS(res[0],window,i);
			c = getSubBIS(res[1],window,i);
			t = getSubBIS(res[2],window,i);
			g = getSubBIS(res[3],window,i);	
			
			fft.realForward(a);
			fft.realForward(c);
			fft.realForward(t);
			fft.realForward(g);
			
			for(int j = 1; j < Math.floor(a.length/2);j++){
				//calculate spectrum values
				double spec_a = Math.sqrt((a[(2*j)+1]*a[(2*j)+1])+(a[(2*j)]*a[(2*j)]));
				double spec_c = Math.sqrt((c[(2*j)+1]*c[(2*j)+1])+(c[(2*j)]*c[(2*j)]));
				double spec_t = Math.sqrt((t[(2*j)+1]*t[(2*j)+1])+(t[(2*j)]*t[(2*j)]));
				double spec_g = Math.sqrt((g[(2*j)+1]*g[(2*j)+1])+(g[(2*j)]*g[(2*j)]));
				//calculate color values
				double c_r = 255%Math.abs((awr*spec_a)+(twr*spec_t)+(cwr*spec_c)+(gwr+spec_g));
				double c_g = 255%Math.abs((awg*spec_a)+(twg*spec_t)+(cwg*spec_c)+(gwg+spec_g));
				double c_b = 255%Math.abs((awb*spec_a)+(twb*spec_t)+(cwb*spec_c)+(gwb+spec_g));

				Color pix = new Color((int)c_r, (int)c_g, (int)c_b);
				image.setRGB(x, y, pix.getRGB());
				x++;
				if(x == sizeofimage){
					x = 0;
					y++;
				}
			}			
		}
		return image;
	}
	public float[] getSubBIS(float[] ar,int window, int offset){
		float[] ret = new float[window];
		try{
			System.arraycopy(ar, offset, ret, 0, window);
		}catch(Exception e){
		}
		return ret;
	}
	public BufferedImage addPixelsToImage(LinkedList<Color >pixels, BufferedImage image){
		int pixelcounter = 0;
		int size = (int) Math.floor(Math.sqrt(pixels.size()));
		image = new BufferedImage(size,size,BufferedImage.TYPE_INT_RGB);
		for(int i = 0; i < image.getHeight();i++){
			for(int j = 0; j < image.getWidth();j++){
				image.setRGB(j, i, pixels.get(pixelcounter).getRGB());
				pixelcounter++;
			}
		}
		return image;
	}
	public BufferedImage bleed(BufferedImage image, int steps){
		Color white = Color.white;
		int height = image.getHeight();
		int width = image.getWidth();
		BufferedImage image2 = image.getSubimage(0,0, width, height);

		for(int i = 0; i < image.getHeight();i++){
			for(int j = 0; j < image.getWidth();j++){
				if(image.getRGB(j,i) == white.getRGB()){
					if(i > 0 && j > 0 && i < image.getHeight()-1 && j < image.getWidth()-1){
						//image2.setRGB(j+1, i, white.getRGB());
						image2.setRGB(j-1, i, white.getRGB());
						//image2.setRGB(1, i+1, white.getRGB());
						image2.setRGB(1, i-1, white.getRGB());
						image2.setRGB(1, i, white.getRGB());
					}
				}
			}
		}
		
		image = null;
		System.gc();
		return image2;
	}
	public BufferedImage threshold(LinkedList<Color> pixels,BufferedImage image,int threshold){
		Color pix;
		int xcounter = 0;
		int ycounter = 0;
		for(int i = 0; i < pixels.size();i++){
			pix = pixels.get(i);
			if(pix.getRed() > threshold){
				pix = new Color(255,pix.getGreen(),pix.getBlue());
			}else{
				pix = new Color(0,pix.getGreen(),pix.getBlue());
			}
			
			if(pix.getGreen() > threshold){
				pix = new Color(pix.getRed(),255,pix.getBlue());
			}else{
				pix = new Color(pix.getRed(),0,pix.getBlue());
			}
			if(pix.getBlue() > threshold){
				pix = new Color(pix.getRed(),pix.getGreen(),255);
			}else{
				pix = new Color(pix.getRed(),pix.getGreen(),0);
			}
			image.setRGB(xcounter, ycounter, pix.getRGB());			
			if(xcounter == image.getWidth()-1){
				xcounter = 0;
				ycounter ++;
			}else{
				xcounter++;
			}
		}
		return image;
		//2736000
		//260690
	}
	public BufferedImage Processlol(BufferedImage image){
		
		float[][] res = generateBISOfWholeSequence();
		//int window = (int) Math.sqrt(res[0].length/2)*10;
		//int window = 120;
		int sequenceSize = res[0].length;
		int numberofwindows = (int) (Math.floor(sequenceSize/2) - (sequenceSize%window));
		int numberofpixels = numberofwindows*120;
		int imageWidth = (int)Math.round(Math.sqrt(numberofpixels)/4);
		int imageHeight = Math.round(imageWidth/2);
		int numberofwindowsx = (int) Math.floor(Math.sqrt(numberofwindows));
		
		
		int sq = (int) Math.sqrt(sequenceSize);
		int xSize = (int) Math.sqrt(sequenceSize-(sequenceSize%window));
		Debugger.Debug("xSize:"+(sequenceSize-window)%window);
		Debugger.Debug("Window size:"+window);
		
		float[] a,c,t,g;
		a=null;
		c=null;
		t=null;
		g=null;
		FloatFFT_1D fft = new FloatFFT_1D(window);
		double awr,awg,awb = 1;
		double cwr,cwg,cwb = 1;
		double twr,twg,twb = 1;
		double gwr,gwg,gwb = 1;
		
		awr = 2;
		cwr = 2;
		twr = 2;
		gwr = 2;
		
		awg = 3;
		cwg = 3;
		twg = 3;
		gwg = 3;
		
		awb = 1;
		cwb = 1;
		twb = 1;
		gwb = 1;
		
		
		
		
		//int imageWidth = Math.round(window/2);
		image = new BufferedImage(imageWidth,imageHeight,BufferedImage.TYPE_INT_RGB);
		boolean lineisfilled = false;
		int pixelcounter = 0;
		boolean hasTmp = false;

		int windowcounter = 0;
		LinkedList tmp = new LinkedList();
		int y = 0;
		for(int i = 0; i < imageHeight; i++){
			if(windowcounter < numberofwindowsx){
			while(!lineisfilled){
				//get window
				if(!hasTmp){
					a = getSubBIS(res[0],window,windowcounter);
					c = getSubBIS(res[1],window,windowcounter);
					t = getSubBIS(res[2],window,windowcounter);
					g = getSubBIS(res[3],window,windowcounter);	
					fft.realForward(a);
					fft.realForward(c);
					fft.realForward(t);
					fft.realForward(g);
					windowcounter ++;
					for(int pe=0; pe < 2; pe++){
						for(int j = 0; j < Math.floor(a.length/2);j++){
							//calculate frequency spectrum
							double spec_a = Math.sqrt(Math.pow(a[(2*j)+1],2)+Math.pow(a[(2*j)],2));
							double spec_c = Math.sqrt(Math.pow(c[(2*j)+1],2)+Math.pow(c[(2*j)],2));
							double spec_t = Math.sqrt(Math.pow(t[(2*j)+1],2)+Math.pow(t[(2*j)],2));
							double spec_g = Math.sqrt(Math.pow(g[(2*j)+1],2)+Math.pow(g[(2*j)],2));
							//calculate color values
							double c_r = Math.abs((awr*spec_a)+(twr*spec_t)+(cwr*spec_c)+(gwr+spec_g));
							double c_g = Math.abs((awg*spec_a)+(twg*spec_t)+(cwg*spec_c)+(gwg+spec_g));
							double c_b = Math.abs((awb*spec_a)+(twb*spec_t)+(cwb*spec_c)+(gwb+spec_g));
							//Debugger.Debug(c_r+"-"+c_g+"-"+c_b);
							//"normalize" values
							if((int)c_r >= 254){
								c_r = 255;
							}
							if((int)c_g >= 254){
								c_g = 255;
							}
							if((int)c_b >= 254){
								c_b = 255;
							}
							//set pixel
							if(j+pixelcounter < imageWidth){
								//Debugger.Debug("SET PIXEL:("+(j+pixelcounter)+","+i+") image size"+imageWidth);
								Color pix = new Color((int)c_r, (int)c_g, (int)c_b);
								//Debugger.Debug("Setting pixel:("+(j+pixelcounter)+","+i+")r:"+c_r+"g:"+c_g+"b:"+c_b);
								image.setRGB(j+pixelcounter, y, pix.getRGB());
								pixelcounter ++;
								
							}
							else{//collect all remaining into tmp
								Color pix = new Color((int)c_r, (int)c_g, (int)c_b);
								tmp.add(pix);
								hasTmp = true;
							}
						}
					}
				}else{//has tmp
					for(int j = 0; j < tmp.size();j++){
						Color pix = (Color) tmp.get(j);
						image.setRGB(j, i, pix.getRGB());
						pixelcounter ++;
						
					}
					tmp = new LinkedList();
					hasTmp = false;
				}
				
				if(pixelcounter >= imageWidth){
					//Debugger.Debug("LINE IS FILLED");
					lineisfilled = true;
					pixelcounter = 0;
					y++;
				}

			}
			lineisfilled = false;
			}
		}
		
		/*
		for(int i = 0; i < imageHeight; i++){
			//get window
			a = getSubBIS(res[0],window,i);
			c = getSubBIS(res[1],window,i);
			t = getSubBIS(res[2],window,i);
			g = getSubBIS(res[3],window,i);
			//fourier
			fft.realForward(a);
			fft.realForward(c);
			fft.realForward(t);
			fft.realForward(g);
			//for(int j = 0; j < Math.floor(a.length/2);j++){
			for(int j = 0; j < numberofwindowsx;j++){
				//calculate spectrum component
				double spec_a = Math.sqrt(Math.pow(a[(2*j)+1],2)+Math.pow(a[(2*j)],2));
				double spec_c = Math.sqrt(Math.pow(c[(2*j)+1],2)+Math.pow(c[(2*j)],2));
				double spec_t = Math.sqrt(Math.pow(t[(2*j)+1],2)+Math.pow(t[(2*j)],2));
				double spec_g = Math.sqrt(Math.pow(g[(2*j)+1],2)+Math.pow(g[(2*j)],2));
				//calculate color value
				double c_r = Math.abs((awr*spec_a)+(twr*spec_t)+(cwr*spec_c)+(gwr+spec_g));
				double c_g = Math.abs((awg*spec_a)+(twg*spec_t)+(cwg*spec_c)+(gwg+spec_g));
				double c_b = Math.abs((awb*spec_a)+(twb*spec_t)+(cwb*spec_c)+(gwb+spec_g));
				//normalize
				if((int)c_r >= 254){
					c_r = 255;
				}
				if((int)c_g >= 254){
					c_g = 255;
				}
				if((int)c_b >= 254){
					c_b = 255;
				}
				//set pixel
				
				Color pix = new Color((int)c_r, (int)c_g, (int)c_b);
				
				image.setRGB(j, i, pix.getRGB());
				
				
			}
			
			
		}
		*/
		return image;
	}

	public BufferedImage Process23(BufferedImage image) {
		
		float[][] res = generateBISOfWholeSequence();//generateBIS();
		float[] a,c,t,g;
		double[] pR,pG,pB;
		double awr,awg,awb = 1;
		double cwr,cwg,cwb = 1;
		double twr,twg,twb = 1;
		double gwr,gwg,gwb = 1;
		
		//Plan p = new Plan(res[0].length,Plan.FORWARD);
		FloatFFT_1D fft = new FloatFFT_1D(res[0].length);
		float[] orig_a = new float[res[0].length];
		float[] orig_c = new float[res[1].length];
		float[] orig_t = new float[res[2].length];
		float[] orig_g = new float[res[3].length];
		System.arraycopy(res[0], 0, orig_a, 0, res[0].length);
		System.arraycopy(res[1], 0, orig_c, 0, res[1].length);
		System.arraycopy(res[2], 0, orig_t, 0, res[2].length);
		System.arraycopy(res[3], 0, orig_g, 0, res[3].length);

		awr = 1;
		cwr = 1;
		twr = 1;
		gwr = 1;
		
		awg = 1;
		cwg = 1;
		twg = 1;
		gwg = 1;
		
		awb = 1;
		cwb = 1;
		twb = 1;
		gwb = 1;
		
		int length = (int) Math.floor(res[0].length/2);
		a = new float[res[0].length];
		c = new float[res[0].length];
		t = new float[res[0].length];
		g = new float[res[0].length];
		res = null;
		System.gc();
		image = new BufferedImage(length,length,BufferedImage.TYPE_INT_RGB);
		for(int i = 0; i < length;i++){
			//copy original
			System.arraycopy(orig_a,0,a,0,orig_a.length);
			System.arraycopy(orig_c,0,c,0,orig_c.length);
			System.arraycopy(orig_t,0,t,0,orig_t.length);
			System.arraycopy(orig_g,0,g,0,orig_g.length);
			//rotate
			a = shiftArray(a);
			c = shiftArray(c);
			t = shiftArray(t);
			g = shiftArray(g);
			//fourier
			fft.realForward(a);
			fft.realForward(c);
			fft.realForward(t);
			fft.realForward(g);
			for(int j = 0; j < length;j++){
				//calculate spectrum component
				double spec_a = Math.sqrt(Math.pow(a[(2*j)+1],2)+Math.pow(a[(2*j)],2));
				double spec_c = Math.sqrt(Math.pow(c[(2*j)+1],2)+Math.pow(c[(2*j)],2));
				double spec_t = Math.sqrt(Math.pow(t[(2*j)+1],2)+Math.pow(t[(2*j)],2));
				double spec_g = Math.sqrt(Math.pow(g[(2*j)+1],2)+Math.pow(g[(2*j)],2));
				//calculate color value
				double c_r = Math.abs((awr*spec_a)+(twr*spec_t)+(cwr*spec_c)+(gwr+spec_g));
				double c_g = Math.abs((awg*spec_a)+(twg*spec_t)+(cwg*spec_c)+(gwg+spec_g));
				double c_b = Math.abs((awb*spec_a)+(twb*spec_t)+(cwb*spec_c)+(gwb+spec_g));
				//normalize
				if(c_r > 255){
					c_r = 255;
				}
				else if(c_g > 255){
					c_g = 255;
				}
				else if(c_b >= 255){
					c_b = 255;
				}
				//set pixel
				try{
				Color pix = new Color((int)c_r, (int)c_g, (int)c_b);
				image.setRGB(j, i, pix.getRGB());
				}catch(Exception e){
					//Debugger.Debug(((int)c_r+":"+(int)c_g+":"+ (int)c_b));
				}
				
				
			}
			
		}
		
		return image;
	}
	
	public BufferedImage Process2(BufferedImage image) {
		
		float[][] res = generateBISOfWholeSequence();//generateBIS();
		float[] a,c,t,g;
		double[] pR,pG,pB;
		double awr,awg,awb = 1;
		double cwr,cwg,cwb = 1;
		double twr,twg,twb = 1;
		double gwr,gwg,gwb = 1;
		
		//Plan p = new Plan(res[0].length,Plan.FORWARD);
		FloatFFT_1D fft = new FloatFFT_1D(res[0].length);
		float[] orig_a = new float[res[0].length];
		float[] orig_c = new float[res[1].length];
		float[] orig_t = new float[res[2].length];
		float[] orig_g = new float[res[3].length];
		System.arraycopy(res[0], 0, orig_a, 0, res[0].length);
		System.arraycopy(res[1], 0, orig_c, 0, res[1].length);
		System.arraycopy(res[2], 0, orig_t, 0, res[2].length);
		System.arraycopy(res[3], 0, orig_g, 0, res[3].length);

		awr = 1;
		cwr = 1;
		twr = 1;
		gwr = 1;
		
		awg = 1;
		cwg = 1;
		twg = 1;
		gwg = 1;
		
		awb = 1;
		cwb = 1;
		twb = 1;
		gwb = 1;
		
		int length = (int) Math.floor(res[0].length/2);
		a = new float[res[0].length];
		c = new float[res[0].length];
		t = new float[res[0].length];
		g = new float[res[0].length];
		res = null;
		System.gc();
		image = new BufferedImage(length,length,BufferedImage.TYPE_INT_RGB);
		for(int i = 0; i < length;i++){
			//copy original
			System.arraycopy(orig_a,0,a,0,orig_a.length);
			System.arraycopy(orig_c,0,c,0,orig_c.length);
			System.arraycopy(orig_t,0,t,0,orig_t.length);
			System.arraycopy(orig_g,0,g,0,orig_g.length);
			//rotate
			a = shiftArray(a);
			c = shiftArray(c);
			t = shiftArray(t);
			g = shiftArray(g);
			//fourier
			fft.realForward(a);
			fft.realForward(c);
			fft.realForward(t);
			fft.realForward(g);
			for(int j = 0; j < length;j++){
				//calculate spectrum component
				double spec_a = Math.sqrt(Math.pow(a[(2*j)+1],2)+Math.pow(a[(2*j)],2));
				double spec_c = Math.sqrt(Math.pow(c[(2*j)+1],2)+Math.pow(c[(2*j)],2));
				double spec_t = Math.sqrt(Math.pow(t[(2*j)+1],2)+Math.pow(t[(2*j)],2));
				double spec_g = Math.sqrt(Math.pow(g[(2*j)+1],2)+Math.pow(g[(2*j)],2));
				//calculate color value
				double c_r = Math.abs((awr*spec_a)+(twr*spec_t)+(cwr*spec_c)+(gwr+spec_g));
				double c_g = Math.abs((awg*spec_a)+(twg*spec_t)+(cwg*spec_c)+(gwg+spec_g));
				double c_b = Math.abs((awb*spec_a)+(twb*spec_t)+(cwb*spec_c)+(gwb+spec_g));
				//normalize
				if(c_r > 255){
					c_r = 255;
				}
				else if(c_g > 255){
					c_g = 255;
				}
				else if(c_b >= 255){
					c_b = 255;
				}
				//set pixel
				try{
				Color pix = new Color((int)c_r, (int)c_g, (int)c_b);
				image.setRGB(j, i, pix.getRGB());
				}catch(Exception e){
					//Debugger.Debug(((int)c_r+":"+(int)c_g+":"+ (int)c_b));
				}
				
				
			}
			
		}
		
		return image;
	}
	private float[][] generateBIS(){
		//10
		LinkedList<GenomeEntity> list = seq.getList();
		int len = list.size()*1;
		float[] a = new float[len];
		float[] c = new float[len];
		float[] t = new float[len];
		float[] g = new float[len];
		float[][] res = new float[4][len];
		
		for(int i = 0; i < list.size(); i++){
			
			if(list.get(i).containsNucleotide(GenomeEntity.NUC_A)){
				a[i] = 1;
			}else if(list.get(i).containsNucleotide(GenomeEntity.NUC_C)){
				c[i] = 1;
			}else if(list.get(i).containsNucleotide(GenomeEntity.NUC_T)){
				t[i] = 1;
			}else if(list.get(i).containsNucleotide(GenomeEntity.NUC_G)){
				g[i] = 1;
			}
		}
		res[0] = a;
		res[1] = c;
		res[2] = t;
		res[3] = g;
		return res;
	}
	private float[][] generateBISOfWholeSequence(){
		LinkedList<GenomeEntity> list = seq.getList();//seq is a class member
		int len = list.size()*10;
		float[] a = new float[len];float[] c = new float[len];float[] t = new float[len];float[] g = new float[len];float[][] res = new float[4][len];
		int counter = 0;
		for(int i = 0; i < list.size(); i++){
			GenomeEntity item = list.get(i);
			String subseq = item.decode();
			for(int j = 0; j < subseq.length();j++){
				char ch = subseq.toLowerCase().charAt(j);
				a[counter] = 0;
				c[counter] = 0;
				t[counter] = 0;
				g[counter] = 0;
				if(ch == 'a'){
					a[counter] = 1;
				}else if(ch == 'c'){
					c[counter] = 1;
				}else if(ch == 't'){
					t[counter] = 1;
				}else if(ch == 'g'){
					g[counter] = 1;
				}
				counter ++;
			}
		}
		res[0] = a;res[1] = c;res[2] = t;res[3] = g;
		return res;
	}
	private float[] shiftArray(float[] ar){
		float tmp = ar[ar.length-1];
		System.arraycopy(ar,0, ar, 1, ar.length-1);
		ar[0] = tmp;
		return ar;
	}

}
